<?php
/**
 * 2007-2023 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2023 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
class WayletRedirectModuleFrontController extends ModuleFrontController
{
    /**
     * Checks before redirecting to he payment gateway
     */
    public function postProcess()
    {
        $config = [
            'WAYLET_COMMERCE_ID' => Configuration::get('WAYLET_COMMERCE_ID', null),
            'WAYLET_API_KEY' => Configuration::get('WAYLET_API_KEY', null),
        ];

        if (empty($config['WAYLET_COMMERCE_ID'])) {
            return $this->displayError($this->module->l('You must provide the Commerce ID in the module settings.'));
        }

        if (empty($config['WAYLET_API_KEY'])) {
            return $this->displayError($this->module->l('You must provide the API key in the module settings.'));
        }

        $module_name = $this->module->name;
        $commerce_id = trim($config['WAYLET_COMMERCE_ID']);
        $api_key = trim($config['WAYLET_API_KEY']);
        $id_cart = Context::getContext()->cart->id;

        if (!$id_cart) {
            return $this->displayError($this->module->l('The cart is empty. Cannot proceed to payment.'));
        }

        $objCart = new Cart($id_cart);
        $total = $objCart->getOrderTotal(true, Cart::BOTH);
        $secure_key = Context::getContext()->customer->secure_key;

        /**
         * Token
         */
        $token = sha1($config['WAYLET_API_KEY'] . $id_cart . $secure_key);

        $url_params_ok = [
            'id_cart' => $id_cart,
            'secure_key' => $secure_key,
            'token' => $token,
        ];
        $url_params_callback = array_merge($url_params_ok, ['callback' => 1]);
        $url_params_ko = array_merge($url_params_ok, ['error' => 1]);

        $data = [
          'externalId' => "{$id_cart}-" . time(),
          'commerceId' => $commerce_id,
          'amount' => $total * 100,
          'callbackUrl' => $this->context->link->getModuleLink($module_name, 'validation', $url_params_callback, true),
          'urlOk' => $this->context->link->getModuleLink($module_name, 'validation', $url_params_ok, true),
          'urlKo' => $this->context->link->getModuleLink($module_name, 'validation', $url_params_ko, true),
        ];

        $url = $this->module->getEnviromentEndpoint();
        $postdata = json_encode($data);

        $headers = [
            'Content-Type: application/json',
            'x-openwaylet-apikey:' . $api_key,
        ];

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $content = curl_exec($ch);
        $err = curl_errno($ch);
        $errmsg = curl_error($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($err !== 0) {
            Logger::addLog("Waylet: cURL error #{$err}: {$errmsg}", 1);
            return $this->displayError($this->module->l('Error trying to connect to Waylet Gateway'));
        }

        switch ($httpcode) {
            case 200:
                $json = json_decode($content);
                if (!is_null($json) && isset($json->url)) {
                    Tools::redirect($json->url);
                    exit;
                }
                break;

            case 400:
                $this->displayError($this->module->l('Request error. Check your Commerce Id in the module´s configuration page.'));
                break;

            case 401:
                $this->displayError($this->module->l('Invalid API key. Check module´s configuration page.'));
                break;

            default:
                Logger::addLog("Waylet: error connecting Waylet Gateway. Http code received: {$httpcode}", 1);
                $this->displayError('Error trying to connect to Waylet Gateway');
                break;
        }
    }

    protected function displayError($message, $description = false)
    {
        $return_btn_link = $this->context->link->getPageLink('order', null, null, 'step=3');
        $return_btn_text = $this->module->l('Go back');

        $this->context->smarty->assign([
            'return_btn_link' => $return_btn_link,
            'return_btn_text' => $return_btn_text,
        ]);

        /*
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message));

        $this->context->smarty->assign(['errors' => $this->errors]);

        return $this->setTemplate('module:waylet/views/templates/front/error.tpl');
    }
}
