<?php
/**
 * 2007-2023 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2023 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
class WayletValidationModuleFrontController extends ModuleFrontController
{
    const WAYLET_STATUS_PAID = 'PAID';

    public function postProcess()
    {
        // Is returning to the error URL? We can stop right here
        $is_error = Tools::getIsset('error');
        if ($is_error) {
            $error_message = $this->module->l('There was an error with your payment');
            Logger::addLog('Waylet: error return URL received', 1);

            return $this->handle_error($error_message, 400);
        }

        // Token and other request checks
        $this->check_request_or_fail();

        // Calback signature
        $is_callback = Tools::getIsset('callback');
        if ($is_callback) {
            Logger::addLog('Waylet: callback request received. Validating signature.', 1);
            $this->check_signature_or_fail();
        }

        // Cart and module checks
        $cart = $this->make_cart_and_module_checks();

        $secure_key = Tools::getValue('secure_key');
        $customer = new Customer((int) $cart->id_customer);

        // Validate the order
        $payment_status = Configuration::get('PS_OS_PAYMENT'); // Default value for a payment that succeed.
        $message = null;
        $module_name = $this->module->displayName;
        $currency_id = (int) Context::getContext()->currency->id;
        $id_cart = $cart->id;

        // Validate order if not already done and if is callback
        $should_validate_order = $is_callback && false == $cart->OrderExists();
        Logger::addLog('Waylet:' . ($should_validate_order ? 'Considering validating order. Checking status.' : 'Not validating order'), 1);

        if ($should_validate_order) {
            $status = $this->get_callback_status();
            $validated_as_paid = $status == self::WAYLET_STATUS_PAID;
            Logger::addLog("Waylet: order with cart ID {$id_cart}. Status received: {$status}", 1);

            if ($validated_as_paid) {
                Logger::addLog("Waylet: validating order with cart ID {$id_cart} as paid", 1);
                $this->module->validateOrder($id_cart, $payment_status, $cart->getOrderTotal(), $module_name, $message, [], $currency_id, false, $secure_key);
            }
        }

        /**
         * If the order has been validated we try to retrieve it
         */
        $order_id = Order::getOrderByCartId((int) $cart->id);

        if ($order_id && ($secure_key == $customer->secure_key)) {
            // Callback response
            if ($is_callback) {
                return $this->respond_json('Order marked as paid', 200);
            }

            // Redirect to confirmation page
            $module_id = $this->module->id;
            Tools::redirect('index.php?controller=order-confirmation&id_cart=' . $id_cart . '&id_module=' . $module_id . '&id_order=' . $order_id . '&key=' . $secure_key);

            return true;
        }

        // Is error
        $error_message = $this->module->l('An error occured. Please contact the merchant to have more information');

        return $this->handle_error($error_message, 400);
    }

    /**
     * Check the Prestashop specific params in the request
     */
    private function check_request_or_fail()
    {
        if (
            Tools::getValue('id_cart') == false
            || Tools::getValue('secure_key') == false
            || Tools::getValue('token') == false
        ) {
            $error_message = $this->module->l('Invalid params');

            Logger::addLog('Waylet (check_request_or_fail): PS params not present ', 1);

            return $this->handle_error($error_message, 403);
        }

        $id_cart = Tools::getValue('id_cart');
        $secure_key = Tools::getValue('secure_key');
        $token_candidate = Tools::getValue('token');

        $token = sha1(Configuration::get('WAYLET_API_KEY', null) . $id_cart . $secure_key);

        $is_invalid = strcmp($token, $token_candidate) !== 0;

        if ($is_invalid) {
            $error_message = $this->module->l('Invalid request');
            Logger::addLog('Waylet (check_request_or_fail): Invalid hash', 1);

            return $this->handle_error($error_message, 403);
        }

        return true;
    }

    /**
     * Checks the received signature data from Waylet to ensure
     * the origin and the request data integrity
     *
     * @return bool
     */
    private function check_signature_or_fail()
    {
        /**
         * The request body is received as JSON
         */
        $request_raw = !empty($_POST) ? $_POST : Tools::file_get_contents('php://input');
        $request_data = json_decode($request_raw, true);
        // Logger::addLog('Waylet (check_signature_or_fail): ' . $request_raw, 1);

        if (is_null($request_data)) {
            Logger::addLog('Waylet (check_signature_or_fail): Error with json conversion', 1);

            return;
        }

        $required_fields = ['paymentId', 'wayletPaymentId', 'status', 'amount', 'signature'];
        $has_all_required_fields = true;

        // Check if all required fields are present
        foreach ($required_fields as $field) {
            if (!isset($request_data[$field])) {
                $has_all_required_fields = false;
                break;
            }
        }

        if (false === $has_all_required_fields) {
            Logger::addLog('Waylet (check_signature_or_fail): Required parameters not present', 1);

            return false;
        }

        // Secret, stored in module´s config
        $secret = Configuration::get('WAYLET_SECRET', null);

        // Convert input data to expected types
        $payment_id = (string) $request_data['paymentId'];
        $waylet_payment_id = (string) $request_data['wayletPaymentId'];
        $status = (string) trim($request_data['status']);
        $amount = (string) $request_data['amount'];
        $signature = (string) $request_data['signature'];

        // Create signature
        $local_hash = hash_hmac('sha512', $payment_id . $waylet_payment_id . $status . $amount, $secret);

        // Compare signatures in a timing-attack safe way
        $is_valid_signature = hash_equals($local_hash, $signature);

        if (false === $is_valid_signature) {
            $error_message = $this->module->l('Invalid request signature');
            Logger::addLog('Waylet (check_signature_or_fail): Invalid request signature', 1);

            return $this->handle_error($error_message, 403);
        }

        return $is_valid_signature;
    }

    /**
     * Returns the callback status parameter received
     * @return string
     */
    private function get_callback_status()
    {
        $request_raw = !empty($_POST) ? $_POST : Tools::file_get_contents('php://input');
        $request_data = json_decode($request_raw, true);

        $status = (string) trim($request_data['status']);

        return $status;
    }

    /**
     * Make the proper checks to the order (Cart at this point)
     *
     * @return Cart
     */
    private function make_cart_and_module_checks()
    {
        $id_cart = Tools::getValue('id_cart');
        $cart = new Cart((int) $id_cart);

        if (!\Validate::isLoadedObject($cart)) {
            Logger::addLog('Waylet (make_cart_and_module_checks): Cart not found', 1);

            return $this->handle_error($this->module->l('Cart not found'), 400);
        }

        /*
         * Some order and module checks
         */
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Logger::addLog('Waylet (make_cart_and_module_checks): Error with cart', 1);
            Tools::redirect('index.php?controller=order&step=1');
        }

        return $cart;
    }

    /**
     * Handle error considering if is a direct callback or a user´s return
     *
     * @param string $error_message
     * @param int $http_code Desired status code
     */
    private function handle_error($error_message, $http_code = 400)
    {
        $is_callback = Tools::getIsset('callback');
        if ($is_callback) {
            return $this->respond_json($error_message, $http_code);
        }

        // If is not callback, show error screen to the user
        //$this->errors[] = $error_message;

        $waylet_logo_url = $this->module->getPathUri() . 'views/img/logo-admin.png';
        $return_btn_link = $this->context->link->getPageLink('order', null, null, 'step=3');

        $this->context->smarty->assign([
            'waylet_logo_url' => $waylet_logo_url,
            'errors' => [$error_message],
            'return_btn_link' => $return_btn_link
        ]);

        return $this->setTemplate('module:waylet/views/templates/front/error.tpl');
    }

    /**
     * Return an HTTP JSON response
     *
     * @param mixed $response_data
     * @param int $http_code
     *
     * @return json
     */
    private function respond_json($response_data, $http_code = 200)
    {
        header('Content-Type: application/json; charset=utf-8');
        http_response_code($http_code);
        $this->ajaxDie(Tools::jsonEncode($response_data));
    }
}
