<?php
/**
 * 2007-2023 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2023 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class Waylet extends PaymentModule
{
    protected $config_form = false;

    // Gateway constants
    const ENVIROMENT_PREPRODUCTION = 'preproduction';
    const ENVIROMENT_PRODUCTION = 'production';
    const DEFAULT_ENVIROMENT = self::ENVIROMENT_PREPRODUCTION;
    const ENDPOINT_PREPRODUCTION = 'https://pre.wayletlabs.com/api/openwaylet/web/v1/transactions';
    const ENDPOINT_PRODUCTION = 'https://pro.waylet.es/api/openwaylet/web/v1/transactions';

    public function __construct()
    {
        $this->name = 'waylet';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.2';
        $this->author = 'Waylet';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->module_key = '31e9f3ef99175f72c2712a963eb7d5fd';
        $this->displayName = $this->l('Pay with Waylet');
        $this->description = $this->l('Allow your clients to pay using Waylet, the Repsol´s app.');
        $this->limited_currencies = ['EUR'];
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];

        parent::__construct();
    }

    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');

            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');

            return false;
        }

        // Default enviroment
        Configuration::updateValue('WAYLET_ENVIROMENT', self::DEFAULT_ENVIROMENT);

        return parent::install()
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('payment')
            && $this->registerHook('paymentReturn')
            && $this->registerHook('paymentOptions')
            && $this->registerHook('displayPaymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('WAYLET_COMMERCE_ID');
        Configuration::deleteByName('WAYLET_API_KEY');
        Configuration::deleteByName('WAYLET_SECRET');
        Configuration::deleteByName('WAYLET_ENVIROMENT');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /*
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('submitWayletModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('is_production_enviroment', $this->isProductionEnviroment());

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitWayletModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return [
            'form' => [
                'legend' => [
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 3,
                        'type' => 'select',
                        'desc' => $this->l('Select "Preproduction" to test the integration, or "Production" to allow real payments'),
                        'name' => 'WAYLET_ENVIROMENT',
                        'label' => $this->l('Mode'),
                        'required' => true,
                        'options' => [
                            'query' => [
                                ['key' => self::ENVIROMENT_PREPRODUCTION, 'name' => $this->l('Pre-production (Test)')],
                                ['key' => self::ENVIROMENT_PRODUCTION, 'name' => $this->l('Production (Real)')],
                            ],
                            'id' => 'key',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-info"></i>',
                        'desc' => $this->l('Enter your Commerce ID'),
                        'required' => true,
                        'name' => 'WAYLET_COMMERCE_ID',
                        'label' => $this->l('Commerce ID'),
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'name' => 'WAYLET_API_KEY',
                        'label' => $this->l('API key'),
                        'required' => true,
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'name' => 'WAYLET_SECRET',
                        'label' => $this->l('Secret'),
                        'required' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return [
            'WAYLET_COMMERCE_ID' => Configuration::get('WAYLET_COMMERCE_ID', null),
            'WAYLET_API_KEY' => Configuration::get('WAYLET_API_KEY', null),
            'WAYLET_ENVIROMENT' => Configuration::get('WAYLET_ENVIROMENT', self::DEFAULT_ENVIROMENT),
            'WAYLET_SECRET' => Configuration::get('WAYLET_SECRET', null),
        ];
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Checks if the enviroment is set to preproduction (test enviroment)
     *
     * @return bool
     */
    public function isTestEviroment()
    {
        return !$this->isProductionEnviroment();
    }

    /**
     * Checks if the enviroment is set to production (real payments)
     *
     * @return bool
     */
    public function isProductionEnviroment()
    {
        return Configuration::get('WAYLET_ENVIROMENT', self::DEFAULT_ENVIROMENT) == self::ENVIROMENT_PRODUCTION;
    }

    /**
     * Returns the endpoint based on the enviroment configured
     *
     * @return string
     */
    public function getEnviromentEndpoint()
    {
        if ($this->isProductionEnviroment()) {
            return self::ENDPOINT_PRODUCTION;
        }

        return self::ENDPOINT_PREPRODUCTION;
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
            $this->context->controller->addJs($this->_path . 'views/js/back.js');
        }
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        $order = $params['order'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
            $this->smarty->assign('status', 'ok');
        }

        $this->smarty->assign([
            'id_order' => $order->id,
            'reference' => $order->reference,
            'params' => $params,
            'total' => Tools::displayPrice($order->getTotalPaid()),
        ]);

        return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option
            ->setCallToActionText($this->l('Pay with Waylet'))
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', [], true))
            ->setLogo($this->context->link->getMediaLink($this->_path . 'views/img/logo-payment-option.png'))
        ;

        return [
            $option,
        ];
    }

    /**
     * Make the currency checks
     *
     * @param \Cart $cart
     *
     * @return bool
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }
}
